/*
 * @author lsy
 * @date   2020/6/29
 **/
import 'dart:async';
import 'dart:math';
import 'dart:ui';

import 'package:extended_nested_scroll_view/extended_nested_scroll_view.dart'
    as extend;
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_boost/flutter_boost.dart';
import 'package:flutter_common/commonModel/picker/base/BaseBottomPicker.dart';
import 'package:flutter_screenutil/screenutil.dart';
import 'package:gm_flutter/ClueModel/page/DiscussLowPrice/DiscussLowPricePopView.dart';
import 'package:gm_flutter/ClueModel/page/common/StickyTabBarDelegate.dart';
import 'package:gm_flutter/ClueModel/page/levelOne/LevelOneList.dart';
import 'package:gm_flutter/ClueModel/page/levelOne/LevelOneModel.dart';
import 'package:gm_flutter/ClueModel/page/mechanismBox/MechanismBoxPage.dart';
import 'package:gm_flutter/ClueModel/server/entity/PlanBarBean.dart';
import 'package:gm_flutter/ClueModel/util/NumPlanUtil.dart';
import 'package:gm_flutter/commonModel/App.dart';
import 'package:gm_flutter/commonModel/GMBase.dart';
import 'package:gm_flutter/commonModel/base/BaseComponent.dart';
import 'package:gm_flutter/commonModel/base/BaseState.dart';
import 'package:gm_flutter/commonModel/bean/Pair.dart';
import 'package:gm_flutter/commonModel/cache/CacheManager.dart';
import 'package:gm_flutter/commonModel/eventbus/SimpleEventBus.dart';
import 'package:gm_flutter/commonModel/view/baseRefreshIndicator.dart';
import 'package:gm_flutter/main.mark.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';

class LevelOnePage extends StatefulWidget {
  int planId = 10;
  Map<String, dynamic> params;
  final String PAGE_NAME = "level_one_plan_detail";
  String title;

  LevelOnePage(this.params) {
    this.planId = params["planId"];
    this.title = params["title"];
  }

  @override
  State<StatefulWidget> createState() => LevelOneState();
}

class LevelOneState extends BaseState<LevelOnePage>
    with TickerProviderStateMixin {
  LevelOneModel _model = new LevelOneModel();
  RefreshController _refreshController = RefreshController();
  GlobalKey keyTop = new GlobalKey();
  GlobalKey key1 = new GlobalKey();
  double screenWidth;
  List<Widget> oneList = new List();
  int lastPageIndex = 0;

  TabController tabController;

  int index = 0;

  double stateBarHeight = 0;

  @override
  void initState() {
    super.initState();
    _model.plan_id = widget.planId;
    _model.init(() {
      _initTabBar();
    });
  }

  void _initTabBar() {
    if (tabController != null) {
      tabController.removeListener(tabControlerListener);
    }
    tabController = TabController(
        initialIndex: index, length: _model.tabsList.length, vsync: this);
    tabController.addListener(tabControlerListener);
    setState(() {});
  }

  @override
  void dispose() {
    tabController?.removeListener(tabControlerListener);
    tabController?.dispose();
    _model.dispose();
    _refreshController.dispose();
    super.dispose();
  }

  void tabControlerListener() {
    if (index != tabController.index) {
      index = tabController.index;
    }
  }

  @override
  Widget buildItem(BuildContext context) {
    stateBarHeight = MediaQueryData.fromWindow(window).padding.top;

    oneList.clear();
    oneList.add(good());
    oneList.add(rect());
    oneList.add(explain());
    oneList.add(Container(
      height: 5,
      color: Color(0xffF7F6FA),
    ));
    screenWidth = MediaQuery.of(context).size.width;
    return Scaffold(
        backgroundColor: Colors.white,
        body: SafeArea(
            top: false,
            child: baseStateView(
                MediaQuery.of(context).size.width,
                MediaQuery.of(context).size.height,
                _model.stateLive,
                Container(
                    child: Column(
                  children: <Widget>[
                    Expanded(
                      child: BaseNestedScrollViewRefreshIndicator(
                          onRefresh: () async {
                            Completer completer = new Completer();
                            _model.init(() {
                              _initTabBar();
                              SimpleEventBus.instance()
                                  .notifyListener("LevelOneList", "");
                              completer.complete();
                            });
                            return completer.future;
                          },
                          child: newHomeWarp()),
                    ),
                    Container(
                      height: 0.5,
                      width: double.maxFinite,
                      color: Color(0xffE5E5E5),
                    ),
                    bottomWidget(),
                  ],
                )), () {
              _model.stateLive.notifyView(LOADING);
              _model.init(() {
                _initTabBar();
              });
            }, paddingTop: stateBarHeight + kToolbarHeight)));
  }

  Widget newHomeWarp() {
    return Stack(
      children: <Widget>[
        Container(
            width: MediaQuery.of(context).size.width,
            height: MediaQuery.of(context).size.height,
            child: NotificationListener(
              onNotification: (scrollNotification) {
                if (scrollNotification is KeepAliveNotification ||
                    scrollNotification is OverscrollIndicatorNotification) {
                  return false;
                }
                if (scrollNotification is ScrollUpdateNotification &&
                    scrollNotification.metrics.axisDirection.index == 2) {}
                if (scrollNotification.depth == 0) {
                  if (scrollNotification.metrics.pixels > 80) {
                    _model.textLive.notifyView(
                        (scrollNotification.metrics.pixels - 80) / 40 > 1.0
                            ? 1.0
                            : (scrollNotification.metrics.pixels - 80) / 40);
                  } else {
                    _model.textLive.notifyView(0.0);
                  }
                } else if (scrollNotification.depth == 2) {
                  if (scrollNotification.metrics.pixels < 50 &&
                      scrollNotification.metrics.pixels > 0) {
                    _model.textLive.notifyView(1.0);
                  }
                }
                return false;
              },
              child: newHome(),
            )),
        baseSliverTitle("${widget.title}", MediaQuery.of(context).size.width,
            _model.textLive, context),
        baseSliverBack(() {
          if (RouterCenterImpl().findMainRouter().isWithNative()) {
            FlutterBoost.singleton.closeCurrent();
          } else {
            Navigator.of(context).pop();
          }
        }, context),
      ],
    );
  }

  Widget newHome() {
    List<Widget> list = [];
    for (int i = 0; i < _model.tabsList.length; i++) {
      list.add(extend.NestedScrollViewInnerScrollPositionKeyWidget(
          Key("Tab${i}"),
          LevelOneList(
              0,
              widget.PAGE_NAME,
              widget.planId,
              _model.tabsList[i].tabType,
              _model.tabsList[i].name,
              kToolbarHeight + stateBarHeight)));
    }
    final double statusBarHeight = MediaQuery.of(context).padding.top;
    final double pinnedHeaderHeight = statusBarHeight + kToolbarHeight;
    return extend.NestedScrollView(
        innerScrollPositionKeyBuilder: () {
          String index = 'Tab${tabController.index.toString()}';
          return Key(index);
        },
        headerSliverBuilder: (BuildContext context, bool innerBoxIsScrolled) {
          return <Widget>[
            SliverOverlapAbsorber(
                handle: extend.NestedScrollView.sliverOverlapAbsorberHandleFor(
                    context),
                sliver: baseSliverAppBar(_model.imageUrl,
                    height: 200)),
            SliverList(
                delegate: SliverChildBuilderDelegate((BuildContext c, int i) {
              if (i == 0) {
                return Container(
                  height: pinnedHeaderHeight,
                );
              }
              return oneList[i - 1];
            }, childCount: oneList.length + 1)),
            SliverPersistentHeader(
              pinned: true,
              delegate: StickyTabBarDelegate(
                  child: Container(
                height: 40,
                color: Colors.white,
                child: baseTabBar(tabController, getTabs(), (index) {
                  _model.selectTab(index, _model.tabsList[index].name,
                      _model.tabsList[_model.currentIndex].name);
                  _model.currentIndex = index;
                }, scroll: false),
              )),
            ),
          ];
        },
        body: Container(
            color: Color(0xffF7F6FA),
            child: TabBarView(controller: tabController, children: list)));
  }

  List<Widget> getTabs() {
    List<Widget> list = [];
    for (int i = 0; i < _model.tabsList.length; i++) {
      list.add(baseTabBarItem(_model.tabsList[i].name));
    }
    return list;
  }

  Widget head() {
    return Container(
      key: keyTop,
      width: double.maxFinite,
      height: 200,
      alignment: Alignment.topLeft,
      decoration: BoxDecoration(
          image: DecorationImage(
              fit: BoxFit.cover,
              image: NetworkImage(
                  "https://pic.igengmei.com/2018/09/11/1513/b7e825a4e4c1-w"))),
//      child: ,
    );
  }

  Widget good() {
    if (_model.planoverItem == null) {
      return Container(
        height: 54,
      );
    }
    return Container(
      width: MediaQuery.of(context).size.width,
      height: 54,
      margin: EdgeInsets.only(top: 2, bottom: 12),
      child: Stack(
        children: <Widget>[
          Positioned(
            top: 6,
            left: 15,
            child: baseText(_model.planoverItem.name, 18, Color(0xff282828),
                bold: true),
          ),
          Positioned(
            bottom: 3.5,
            left: 15,
            child: Container(
              width: MediaQuery.of(context).size.width - 30 - 100,
              child: baseText("${_model.planoverItem.planDescription}", 12,
                  Color(0xff999999)),
            ),
          ),
          Positioned(
            right: 0,
            child: Container(
              width: 100,
              height: 54,
              decoration: BoxDecoration(
                  gradient: LinearGradient(colors: [
                Color(0xFFFFF0F5),
                Color(0xffFFF8FA),
                Color(0xFFFFFFFF)
              ], begin: FractionalOffset(1, 0), end: FractionalOffset(0, 1))),
            ),
          ),
          Positioned(
            right: 15,
            top: 6,
            child: Row(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.baseline,
              textBaseline: TextBaseline.alphabetic,
              children: <Widget>[
                baseText("好评率", 11, Color(0xff282828)),
                Container(
                  margin: EdgeInsets.only(left: 4),
                  child: baseText(
                      "${(_model.planoverItem.positiveRate.contains(".") ? _model.planoverItem.positiveRate.split(".")[0] : _model.planoverItem.positiveRate).replaceAll("%", "")}",
                      20,
                      Color(0xffFF5963)),
                ),
                baseText("%", 11, Color(0xffFF5963)),
              ],
            ),
          ),
          Positioned(
            right: 15,
            bottom: 8,
            child: baseText(
                "销量${_model.planoverItem.salesCount}", 11, Color(0xff666666)),
          )
        ],
      ),
    );
  }

  Widget rect() {
    if (_model.planoverItem == null) {
      return Container(
        height: 62,
      );
    }
    List<Widget> list = List();
    int i = 0;
    _model.planoverItem.overviewAttrs.forEach((element) {
      list.add(Expanded(
        child: Container(
          alignment: Alignment.center,
          child: Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: <Widget>[
              Container(
                margin: EdgeInsets.only(left: 2, right: 2),
                child: baseText(element.attrValue, 14, Color(0xff282828),
                    bold: true),
              ),
              Container(
                margin: EdgeInsets.only(left: 2, right: 2),
                child: Text(
                  element.attrName,
                  textScaleFactor: 1.0,
                  maxLines: 1,
                  overflow: TextOverflow.ellipsis,
                  style: TextStyle(
                      decoration: TextDecoration.none,
                      fontSize: 11,
                      color: Color(0xff999999),
                      fontStyle: FontStyle.normal,
                      fontWeight: FontWeight.w400),
                ),
              )
            ],
          ),
        ),
      ));
      if (i != _model.planoverItem?.overviewAttrs.length - 1) {
        list.add(Container(
          width: 0.5,
          height: 18,
          color: Color(0xFFE5E5E5),
        ));
      }
      i++;
    });
    return Container(
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(4),
        color: Color(0xffF9F8FB),
      ),
      width: double.maxFinite,
      height: 62,
      margin: EdgeInsets.only(left: 15, right: 15, bottom: 20),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: list,
      ),
    );
  }

  Widget explain() {
    List<Widget> list = [];
    list.add(Row(
      children: <Widget>[
        baseText("项目说明", 15, Color(0xff282828), bold: true),
        Expanded(
          child: Container(),
        ),
        GestureDetector(
          onTap: () {
            Map<String, dynamic> map = {
              "business_id": "${widget.planId}",
              "planId": widget.planId,
              "referrer_tab_name": _model.tabsList[tabController.index].name
            };
            RouterCenterImpl()
                .findMainRouter()
                .jumpPage(context, "level_one_plan_instruction", map, false);
          },
          behavior: HitTestBehavior.opaque,
          child: baseText("了解更多", 12, Color(0xff3FB5AF)),
        )
      ],
    ));
    list.add(Container(
      height: 11,
    ));
    if (_model.planoverItem != null) {
      _model.planoverItem.explanationAttrs.forEach((element) {
        List<Widget> textList = [];
        if (element.attrName.length < 5) {
          for (int i = 0; i < element.attrName.length; i++) {
            textList.add(baseText(
                element.attrName.substring(i, i + 1), 13, Color(0xff999999)));
            if (i < element.attrName.length - 1) {
              textList.add(Expanded(
                child: Container(),
              ));
            }
          }
        }
        list.add(Container(
          margin: EdgeInsets.only(bottom: 10),
          child: Row(
            children: <Widget>[
              Container(
                  width: 55,
                  child: element.attrName.length < 5
                      ? Row(
                          children: textList,
                        )
                      : baseText(element.attrName, 13, Color(0xff999999))),
              Expanded(
                child: Container(
                  margin: EdgeInsets.only(left: 12),
                  child: Text(
                    element.attrValue,
                    textScaleFactor: 1.0,
                    softWrap: false,
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(
                        decoration: TextDecoration.none,
                        fontSize: 13,
                        color: Color(0xff666666),
                        fontStyle: FontStyle.normal,
                        fontWeight: FontWeight.w400),
                  ),
                ),
              )
            ],
          ),
        ));
      });
    }
    return Container(
      margin: EdgeInsets.only(left: 15, right: 15, bottom: 9),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: list,
      ),
    );
  }

  Widget bottomWidget() {
    return Container(
      width: double.maxFinite,
      height: 55,
      color: Colors.white,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: <Widget>[
          GestureDetector(
            behavior: HitTestBehavior.opaque,
            onTap: () {
              Map<String, dynamic> map = {
                "business_id": "${widget.planId}",
                "planId": widget.planId,
                "planType": 1,
                "referrer_tab_name": _model.tabsList[tabController.index].name
              };
              RouterCenterImpl()
                  .findMainRouter()
                  .jumpPage(context, "level_one_plan_compare", map, false);
            },
            child: Container(
              margin: EdgeInsets.only(left: 21, right: 20),
              width: 30,
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Container(
                    width: 22,
                    height: 22,
                    child: Image.asset("assets/vs_black.png"),
                  ),
                  Container(
                    margin: EdgeInsets.only(top: 3),
                    child: baseText("去比较", 10, Color(0xff282828)),
                  )
                ],
              ),
            ),
          ),
          Expanded(
              child: GestureDetector(
                  behavior: HitTestBehavior.opaque,
                  onTap: () {
                    if (CacheManager.getInstance()
                                .get(MEMORY_CACHE)
                                .get(USER_ID) ==
                            null &&
                        RouterCenterImpl().findMainRouter().isWithNative()) {
                      RouterCenterImpl()
                          .findMainRouter()
                          .jumpPage(context, "show_login", null, true);
                      return;
                    }
                    BaseBottomPicker()
                      ..setPicker(MechanismBox(widget.planId, widget.PAGE_NAME))
                      ..show(context);
                  },
                  child: Container(
                    height: 40,
                    decoration: BoxDecoration(
                        color: Color(0xff51CDC7),
                        borderRadius: BorderRadius.circular(20)),
                    alignment: Alignment.center,
                    child: baseText("咨询", 14, Colors.white, bold: true),
                  ))),
          Container(
            width: 15,
          ),
          Expanded(
              child: GestureDetector(
                  behavior: HitTestBehavior.opaque,
                  onTap: () {
                    if (CacheManager.getInstance()
                                .get(MEMORY_CACHE)
                                .get(USER_ID) ==
                            null &&
                        RouterCenterImpl().findMainRouter().isWithNative()) {
                      RouterCenterImpl()
                          .findMainRouter()
                          .jumpPage(context, "show_login", null, true);
                      return;
                    }
                    DiscussLowPricePopView.showPopView(
                        context, widget.PAGE_NAME, "${widget.planId}", "", "");
                  },
                  child: Container(
                    height: 40,
                    decoration: BoxDecoration(
                        color: Color(0xffF96079),
                        borderRadius: BorderRadius.circular(20)),
                    alignment: Alignment.center,
                    child: baseText("获取底价", 14, Colors.white, bold: true),
                  ))),
          Container(
            width: 15,
          )
        ],
      ),
    );
  }

//  Widget home() {
//    return Column(
//      children: <Widget>[
//        Expanded(
//          child: Stack(
//            children: <Widget>[
//              MediaQuery.removePadding(
//                  removeTop: true,
//                  context: context,
//                  child: NotificationListener(
//                    onNotification: (scrollNotification) {
//                      if (scrollNotification is ScrollUpdateNotification &&
//                          scrollNotification.metrics.axisDirection.index == 2) {
//                        _onScroll(scrollNotification.metrics.pixels);
//                      }
//                      return false;
//                    },
//                    child: baseRefreshView(
//                      _refreshController,
//                          () {
//                        //TODO
////                            _refreshController.refreshCompleted();
//                      },
//                      null,
//                      null,
//                      customScrollView: CustomScrollView(
//                        slivers: <Widget>[
//                          SliverList(
//                            delegate: SliverChildBuilderDelegate(
//                                  (BuildContext context, int index) {
//                                return oneList[index];
//                              },
//                              childCount: oneList.length,
//                            ),
//                          ),
//                        ],
//                      ),
//                      pullUp: true,
//                      onLoading: () {
//                        _model.loadMore();
//                      },
//                    ),
//                  )),
//              StreamBuilder<double>(
//                stream: _model.appBarLive.stream,
//                initialData: _model.appBarLive.data ?? 0.0,
//                builder: (c, data) {
//                  return Opacity(
//                    opacity: data.data,
//                    child: Container(
//                      height: 86,
//                      decoration: BoxDecoration(color: Colors.white),
//                      child: Center(
//                        child: Padding(
//                          padding: EdgeInsets.only(top: 20),
//                          child: Text('文案'),
//                        ),
//                      ),
//                    ),
//                  );
//                },
//              ),
//              baseSliverTitle("title", MediaQuery.of(context).size.width,
//                  _model.appBarLive),
//              Positioned(
//                top: 86,
//                child: StreamBuilder<bool>(
//                  stream: _model.showTab.stream,
//                  initialData: _model.showTab.data ?? false,
//                  builder: (c, data) {
//                    return Visibility(
//                        visible: data.data,
//                        child: Container(
//                          width: MediaQuery.of(context).size.width,
//                          color: Colors.white,
//                          child: MessageBarView(
//                            topIndexLive: _model.topIndexLive,
//                            topScrollLive: _model.topScrollLive,
//                            pageController: pageController,
//                          ),
//                        ));
//                  },
//                ),
//              ),
//              baseSliverBack(() {
//                Navigator.of(context).pop();
//              })
//            ],
//          ),
//        ),
//        Container(
//          height: 0.5,
//          width: double.maxFinite,
//          color: Color(0xffE5E5E5),
//        ),
//        Container(
//          width: double.maxFinite,
//          height: 55,
//          child: Row(
//            mainAxisSize: MainAxisSize.min,
//            crossAxisAlignment: CrossAxisAlignment.center,
//            children: <Widget>[
//              Container(
//                margin: EdgeInsets.only(left: 18),
//                width: 30,
//                child: Column(
//                  mainAxisSize: MainAxisSize.min,
//                  children: <Widget>[
//                    Container(
//                      width: 22,
//                      height: 22,
//                      child: Image.asset("assets/vs_black.png"),
//                    ),
//                    Container(
//                      margin: EdgeInsets.only(top: 3),
//                      child: baseText("去比较", 10, Color(0xff282828)),
//                    )
//                  ],
//                ),
//              ),
//              GestureDetector(
//                  behavior: HitTestBehavior.opaque,
//                  onTap: () {},
//                  child: Container(
//                    margin: EdgeInsets.only(left: 15),
//                    width: 135,
//                    height: 40,
//                    decoration: BoxDecoration(
//                        color: Color(0xff51CDC7),
//                        borderRadius: BorderRadius.circular(20)),
//                    alignment: Alignment.center,
//                    child: baseText("咨询", 14, Colors.white, bold: true),
//                  )),
//              GestureDetector(
//                  behavior: HitTestBehavior.opaque,
//                  onTap: () {},
//                  child: Container(
//                    margin: EdgeInsets.only(left: 15),
//                    width: 135,
//                    height: 40,
//                    decoration: BoxDecoration(
//                        color: Color(0xffF96079),
//                        borderRadius: BorderRadius.circular(20)),
//                    alignment: Alignment.center,
//                    child: baseText("获取底价", 14, Colors.white, bold: true),
//                  )),
//              Expanded(
//                child: Container(),
//              )
//            ],
//          ),
//        )
//      ],
//    );
//  }
}
