import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_boost/flutter_boost.dart';
import 'package:flutter_common/Annotations/anno/RouterCenter.dart';
import 'package:flutter_common/commonModel/util/WindowUtil.dart';
import 'package:flutter_screenutil/screenutil.dart';
import 'package:gm_flutter/main.mark.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'commonModel/base/BaseComponent.dart';
import 'commonModel/base/BaseUtil.dart';
import 'commonModel/cache/CacheManager.dart';
import 'commonModel/nav/NavigationService.dart';
import 'main.mark.dart';

NavigationService navigationService;
bool inProduction = true;

void main() {
  inProduction = bool.fromEnvironment("dart.vm.product");
  print("DEBUG?? ${inProduction}");
  navigationService = NavigationService();
  WidgetsFlutterBinding.ensureInitialized();
  FlutterError.onError = (FlutterErrorDetails details) async {
    if (!inProduction) {
      FlutterError.dumpErrorToConsole(details);
    } else {
      Zone.current.handleUncaughtError(details.exception, details.stack);
    }
  };
  runZonedGuarded(() {
    WidgetsFlutterBinding.ensureInitialized();
    runApp(
      MyAppWidget(),
    );
    SharedPreferences.getInstance().then((value) {
      sharedPreferences = value;
    });
  }, (Object error, StackTrace stack) {
    print("lsy EEEEEE ${error.toString()}   ${stack.toString()}");
    RouterCenterImpl()
        .findMainRouter()
        .catchErrorToBugly("${error.toString()}${stack.toString()}");
  });
}

@RouterCenter()
class MyAppWidget extends StatefulWidget {
  @override
  State<StatefulWidget> createState() {
    return MyApp();
  }
}

class MyApp extends State<MyAppWidget> {
  @override
  void initState() {
    super.initState();
    FlutterBoost.singleton.registerPageBuilders({
      'plan_home': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getPlanPage(params),
      'level_one_plan_detail': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getLevelOnePage(params),
      'level_two_plan_deatil': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getLevelTwoPage(params),
      'level_one_plan_instruction': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getProjectDetailsPage(params),
      'level_two_plan_instruction': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getProjectDetailsPage(params),
      'level_one_plan_compare': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getPlansCompareFeed(params),
      'level_two_plan_compare': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getPlansCompareFeed(params),
      'level_one_plan_compare_detail': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getPlanCompareDetailPage(params),
      'level_two_plan_compare_deatil': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getPlanCompareDetailPage(params),
      'favor_plan': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getTopPage(params),
      'hot_plan': (pageName, params, _) =>
          RouterCenterImpl().findClueRouter().getTopPage(params),
      'proxy': (pageName, params, _) =>
          RouterCenterImpl().findMainRouter().getProxyPage(),
    });
    FlutterBoost.singleton
        .addBoostContainerLifeCycleObserver((state, settings) {
      print("LSY ${state}");
      if (state == ContainerLifeCycle.Init) {
        RouterCenterImpl().findMainRouter().init();
      }
    });
//    FlutterBoost.singleton.addBoostNavigatorObserver(TestBoostNavigatorObserver());
  }

  @override
  Widget build(BuildContext context) {
    WindowUtil.setBarStatus(true);
    return MaterialApp(
        theme: ThemeData(
            primaryColor: Colors.white,
            cursorColor: Color(0xff20BDBB),
            highlightColor: Colors.transparent,
            splashColor: Colors.transparent,
            canvasColor: Colors.white),
        builder: FlutterBoost.init(postPush: _onRoutePushed),
        home: Home());
  }

  void _onRoutePushed(
    String pageName,
    String uniqueId,
    Map<String, dynamic> params,
    Route<dynamic> route,
    Future<dynamic> _,
  ) {}
}

class Home extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => HomeState();
}

class HomeState extends State<Home> {
  @override
  Widget build(BuildContext context) {
    ScreenUtil.init(context, width: 375, height: 667);
    return !inProduction
        ? RouterCenterImpl().findMainRouter().getTestPage()
        : Container(
            color: Colors.white,
            child: Center(
              child: loadingItem(),
            ),
          );
  }
}
